@@ -84,6 +84,9 @@ static char rcsid[] = "$TOG: Label.c /main/26 1997/06/18 17:40:00 samborn $"
 #include "XmI.h"
 #include "XmosI.h"
 #include "XmStringI.h"
+#include <Xm/XpmP.h>
+#include <string.h>
+
 
 #define Pix(w)			((w)->label.pixmap)
 #define Pix_insen(w)		((w)->label.pixmap_insen)
@@ -145,6 +148,10 @@ static char* GetLabelAccelerator(Widget);
 static KeySym GetLabelMnemonic(Widget);
 static XtPointer ConvertToEncoding(Widget, char*, Atom, unsigned long *, 
 				 Boolean *);
+#ifdef FIX_1381
+static Pixmap ConvertToBW(Widget w, Pixmap pm);
+static unsigned int FromColorToBlackAndWhite(char *col);
+#endif
 /********    End Static Function Declarations    ********/
 
 
@@ -688,11 +695,17 @@ SetNormalGC(XmLabelWidget lw)
   lw->label.normal_GC = XtAllocateGC((Widget) lw, 0, valueMask, &values,
 				     dynamicMask, 0);
   
+#ifdef FIX_1381
+/*added for gray insensitive foreground (instead stipple)*/
+  values.foreground =  _XmAssignInsensitiveColor((Widget)lw);
+  values.background = lw->core.background_pixel;
+#else
   valueMask |= GCFillStyle | GCStipple;
   values.foreground = lw->core.background_pixel;
   values.background = lw->primitive.foreground;
   values.fill_style = FillOpaqueStippled;
   values.stipple = _XmGetInsensitiveStippleBitmap((Widget) lw);
+#endif
   
   lw->label.insensitive_GC = XtAllocateGC((Widget) lw, 0, valueMask, &values,
 					  dynamicMask, 0);
@@ -1368,6 +1381,83 @@ Destroy(
   XtReleaseGC ((Widget) lw, lw->label.insensitive_GC);
 }
 
+#ifdef FIX_1381
+static
+unsigned int
+FromColorToBlackAndWhite (char *col)
+{
+    unsigned long r, g, b, bw;
+    char k[5];
+    k[4] = '\0';
+    
+    memcpy(k, col, 4);
+    r = strtoul(k, NULL, 16);
+    memcpy(k, col + 4, 4);
+    g = strtoul(k, NULL, 16);
+    b = strtoul(col + 8, NULL, 16);
+    bw = (0.3 * r + 0.59 * g + 0.11 * b);
+    return (bw);
+}
+
+static
+Pixmap ConvertToBW(Widget w, Pixmap pm)
+{
+    Pixmap bw_pixmap;
+    XpmImage im;
+    int i = 0;
+    unsigned int bw = 0, bw2 = 0;
+    char *col = NULL, *col2 = NULL;
+    
+    XpmCreateXpmImageFromPixmap(XtDisplay(w), pm, 0, &im, NULL);
+    if (im.ncolors > 0)	{
+	if (im.ncolors <= 2) {
+	    if (im.ncolors == 1) {
+		col = strdup(im.colorTable[0].c_color);
+		if (col[0] == '#') {
+		    bw = (FromColorToBlackAndWhite(col + 1) * 0.65);
+		    sprintf(im.colorTable[0].c_color, "#%04x%04x%04x", bw, bw, bw);
+		}
+		if (col) free(col);
+	    } else {
+		col = im.colorTable[0].c_color;
+		col2 = im.colorTable[1].c_color;
+		if ((col[0] == '#') && (col2[0] == '#')) {
+		    bw = FromColorToBlackAndWhite(col + 1);
+		    bw2 = FromColorToBlackAndWhite(col2 + 1);
+		    if (bw >= bw2) {
+			bw2 = bw2 + ((bw - bw2) * 0.65);
+			sprintf(im.colorTable[1].c_color, "#%04x%04x%04x", bw2, bw2, bw2);
+		    } else {
+			bw = bw + ((bw2 - bw) * 0.65);
+			sprintf (im.colorTable[0].c_color, "#%04x%04x%04x", bw, bw, bw);
+		    }
+		}
+	    }
+	} else {
+	    char e[5];
+	    for (i = 0; i < im.ncolors; i++) {
+		col = im.colorTable[i].c_color;
+		if (col[0] == '#') {
+		    bw = FromColorToBlackAndWhite(col + 1);
+		    /* 
+		    could be					
+		    sprintf(im.colorTable[i].c_color, "#%04x%04x%04x", bw, bw, bw);
+		    Four lower lines is sprintf optimized version
+		     */
+		    sprintf(e, "%04x", bw);
+		    memcpy(col + 1, e, 4);
+		    memcpy(col + 5, e, 4);
+		    memcpy(col + 9, e, 4);
+		}
+	    }
+	}
+    }
+    XpmCreatePixmapFromXpmImage(XtDisplay(w), pm, &im, &bw_pixmap, 0, NULL);
+    XpmFreeXpmImage(&im);
+    return(bw_pixmap);
+}
+#endif
+
 /************************************************************************
  *
  *  Redisplay
@@ -1483,8 +1573,11 @@ Redisplay(
 	  Pixmap pix_use = Pix_insen (lw) ;
 
 	  if (pix_use == XmUNSPECIFIED_PIXMAP)
-	      pix_use = Pix(lw);
-
+#ifdef FIX_1381
+		Pix_insen(lw) = pix_use = ConvertToBW(wid, Pix(lw));
+#else
+		pix_use = Pix(lw);
+#endif
 	  if (pix_use != XmUNSPECIFIED_PIXMAP) 
 	    {
 	      gc = lp->insensitive_GC;
@@ -1508,18 +1601,21 @@ Redisplay(
 			    lp->TextRect.y + lp->PixmapRect.y,
 			    1); 
 
+#ifndef FIX_1381
 	      /* if no insensitive pixmap but a regular one, we need
  		 to do the stipple manually, since copyarea doesn't */
  	      if (pix_use == Pix(lw)) {
  		  /* need fill stipple, not opaque */
- 		  XSetFillStyle(XtDisplay(lw), lp->normal_GC, FillStippled);
- 		  XFillRectangle(XtDisplay(lw), XtWindow(lw), lp->normal_GC,
+ 		  XSetFillStyle(XtDisplay(lw), gc, FillStippled);
+ 		  XFillRectangle(XtDisplay(lw), XtWindow(lw), gc,
 			lp->TextRect.x + lp->PixmapRect.x,
 			lp->TextRect.y + lp->PixmapRect.y,
  			lp->PixmapRect.width,
 			lp->PixmapRect.height);
- 		  XSetFillStyle(XtDisplay(lw), lp->normal_GC, FillOpaqueStippled);
+ 		  XSetFillStyle(XtDisplay(lw), gc, FillOpaqueStippled);
+
  	      }
+#endif
 	    }
 	}
     }
@@ -1557,6 +1653,9 @@ Redisplay(
 		       lp->StringRect.width,
 		       lp->alignment,
 		       XmPrim_layout_direction(lw), NULL);
+
+#ifdef FIX_1381
+#else
 #ifdef USE_XFT
       if (!XtIsSensitive(wid))
         {
@@ -1569,6 +1668,8 @@ Redisplay(
           XSetFillStyle(XtDisplay(lw), lp->insensitive_GC, FillOpaqueStippled);
         }
 #endif
+#endif
+
     }
   
   if (lp->_acc_text != NULL) 
@@ -2831,9 +2932,11 @@ ConvertToEncoding(Widget w, char* str, Atom encoding,
     /* Fix for Bug 1117 - if str is null then a SEGVIOLATION occures
      * in strlen.
      */
+    *length = str ? strlen(rval) : 0;
+
     rval = _XmTextToLocaleText(w, (XtPointer) str,
 			       COMPOUND_TEXT, 8, 
-			       (str)?strlen(str):0,
+			       *length,
 			       flag);
   }
 
@@ -2919,7 +3022,7 @@ LabelGetValue(Widget w, int type)
 {
   XmString value;
   
-  XtVaGetValues(w, XmNlabelString, &value, 0);
+  XtVaGetValues(w, XmNlabelString, &value, NULL);  
   
   if (type == XmFORMAT_XmSTRING) 
     {
