@@ -2524,6 +2524,7 @@ processCharsetAndTextUtf8(XmStringCharSet tag,
   /* Now copy in the text */
   if (ctlen > 0) {
     char *text = Convert(ctext, ctlen, "UTF-8", tag);
+    if (text == NULL) return(False);
     *outc = ctextConcat(*outc, *outlen, text, strlen(text));
     *outlen += ctlen;
     XtFree(text);
@@ -2554,6 +2555,9 @@ processCharsetAndText(XmStringCharSet tag,
 		      ct_Charset	*prev)
 {
   unsigned int		ctlen = 0, len;
+  
+  if (strcmp(tag, CS_UTF_8) == 0)
+    tag = XmFONTLIST_DEFAULT_TAG;
 
   if (strcmp(tag, XmFONTLIST_DEFAULT_TAG) == 0)
     {
