@@ -97,6 +97,7 @@ extern "C" { /* some 'locale.h' do not have prototypes (sun) */
   (((tablist) != NULL) && \
    ((unsigned int)(unsigned long)(tablist) != XmAS_IS))
 
+#define  FIX_1414
 /**********************************************************************
  *	      IMPORTANT NOTE: IMPLEMENTATION OF SHARING
  *
@@ -1977,6 +1978,38 @@ ValidateTag(XmRendition rend,
     }
 }
 
+#ifdef FIX_1414
+static int
+GetSameRenditions(XmRendition *rend_cache, XmRendition rend, int count_rend)
+{
+	int i;
+	for (i=0; i<count_rend; i++){
+
+		if ( rend_cache && (rend_cache[i])
+				&& (((_XmRendFontName(rend) 	&& _XmRendFontName(rend_cache[i]) ) && !strcmp(_XmRendFontName(rend_cache[i]), _XmRendFontName(rend))
+					|| 	(!_XmRendFontName(rend) && !_XmRendFontName(rend_cache[i])))
+				&& (((_XmRendFontFoundry(rend) && _XmRendFontFoundry(rend_cache[i])) && !strcmp(_XmRendFontFoundry(rend_cache[i]), _XmRendFontFoundry(rend)))
+					|| 	(!_XmRendFontFoundry(rend) && !_XmRendFontFoundry(rend_cache[i])))
+				&& (((_XmRendFontEncoding(rend) && _XmRendFontEncoding(rend_cache[i])) && !strcmp(_XmRendFontEncoding(rend_cache[i]), _XmRendFontEncoding(rend)))
+					|| 	(!_XmRendFontEncoding(rend) && !_XmRendFontEncoding(rend_cache[i])))
+				&& (((_XmRendFontStyle(rend) && _XmRendFontStyle(rend_cache[i])) && !strcmp(_XmRendFontStyle(rend_cache[i]), _XmRendFontStyle(rend)))
+					|| 	(!_XmRendFontStyle(rend) && !_XmRendFontStyle(rend_cache[i])) )
+				&& _XmRendFontSize(rend) == _XmRendFontSize(rend_cache[i])
+				&& _XmRendPixelSize(rend) == _XmRendPixelSize(rend_cache[i])
+				&& _XmRendFontSlant(rend) == _XmRendFontSlant(rend_cache[i])
+				&& _XmRendFontWeight(rend) == _XmRendFontWeight(rend_cache[i])
+				&& _XmRendFontSpacing(rend) == _XmRendFontSpacing(rend_cache[i]))
+	   	   )
+		{
+			return i;
+		}
+
+	}
+	return -1;
+
+}
+#endif
+
 /* Make sure all the font related resources make sense together and */
 /* then load the font specified by fontName if necessary. */
 static void
@@ -2042,22 +2075,32 @@ ValidateAndLoadFont(XmRendition rend, Display *display)
 #ifdef USE_XFT
 		case XmFONT_IS_XFT:
 		  {
-		    XftResult res;
-		    XftPattern *p;
+		    FcResult res;
+		    FcPattern *p;
 		    
+#ifdef FIX_1414
+						  static XmRendition *rend_cache;
+						  static int count_rend=0, num_rend;
+						  num_rend = GetSameRenditions(rend_cache, rend, count_rend);
+
+						  if (num_rend>=0)
+							  _XmRendXftFont(rend) = _XmRendXftFont(rend_cache[num_rend]);
+						  else
+						  {
+#endif
 		    _XmRendPattern(rend) = FcPatternCreate();
 		    if (_XmRendFontName(rend))
 		      FcPatternAddString(_XmRendPattern(rend), FC_FAMILY,
-		                         _XmRendFontName(rend));
+		                         (XftChar8 *)_XmRendFontName(rend));
 		    if (_XmRendFontFoundry(rend))
 		      FcPatternAddString(_XmRendPattern(rend), FC_FOUNDRY,
-		                         _XmRendFontFoundry(rend));
+		                         (XftChar8 *)_XmRendFontFoundry(rend));
 		    if (_XmRendFontEncoding(rend))
 		      FcPatternAddString(_XmRendPattern(rend), XFT_ENCODING,
-		                         _XmRendFontEncoding(rend));
+		                         (XftChar8 *)_XmRendFontEncoding(rend));
 		    if (_XmRendFontStyle(rend))
 		      FcPatternAddString(_XmRendPattern(rend), FC_STYLE,
-		                         _XmRendFontStyle(rend));
+		                         (XftChar8 *)_XmRendFontStyle(rend));
 		    if (_XmRendFontSize(rend))
 		      FcPatternAddInteger(_XmRendPattern(rend), FC_SIZE,
 		                         _XmRendFontSize(rend));
@@ -2074,7 +2117,17 @@ ValidateAndLoadFont(XmRendition rend, Display *display)
 		      FcPatternAddInteger(_XmRendPattern(rend), FC_SPACING,
 		                         _XmRendFontSpacing(rend));
                     p = XftFontMatch(display, 0, _XmRendPattern(rend), &res);
+#ifdef FIX_1414
                     _XmRendXftFont(rend) = XftFontOpenPattern(display, p);
+					    		  rend_cache = (XmRendition *) XtRealloc((char *)rend_cache, 
+					    		    (Cardinal)(sizeof(XmRendition) * (count_rend + 1)));
+							  rend_cache[count_rend] =_XmRenditionCopy(rend, TRUE);
+							  count_rend++;
+						  }
+
+#else
+						  _XmRendXftFont(rend) = XftFontOpenPattern(display, p);
+#endif
 		  }
 		  result = _XmRendXftFont(rend) != NULL;
 		  break;
@@ -2924,7 +2977,7 @@ _XmXftDrawString2(Display *display, Window window, GC gc, XftFont *font, int bpc
     {
 	case 1:
 		XftDrawStringUtf8(draw, &xftcol, font,
-			x, y, s, len);
+			x, y, (XftChar8 *)s, len);
 		break;
 	case 2:
 		XftDrawString16(draw, &xftcol, font,
@@ -3020,7 +3073,7 @@ _XmXftDrawString(Display *display, Window window, XmRendition rend, int bpc,
     {
 	case 1:
 		XftDrawStringUtf8(draw, &fg_color, _XmRendXftFont(rend),
-			x, y, s, len);
+			x, y, (XftChar8 *)s, len);
 		break;
 	case 2:
 		XftDrawString16(draw, &fg_color, _XmRendXftFont(rend),
@@ -3059,6 +3112,21 @@ _XmXftGetXftColor(Display *display, Pixel color)
     xftcol.color.alpha = 0xFFFF;
     return xftcol;
 }
+
+#ifdef FIX_1415
+void _XmXftFontAverageWidth(Widget w, XtPointer f, int *width)
+{
+	XftFont *fp = (XftFont *)f;
+	static char	*s = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
+	int l = 62; /* strlen(s) */
+	XGlyphInfo	ext;
+
+	XftTextExtents8(XtDisplay(w), fp, (unsigned char *)s, l, &ext);
+    if (width) 
+    	*width = ext.width / l;
+}
+#endif
+
 #endif
 
 /*ARGSUSED*/
